@@ -1,4 +1,4 @@
+"""Allows import of Terrain Engine classes"""
 
-# populate module namespace
 from fcpgtools.terrainengine.taudem_engine import TauDEMEngine
 from fcpgtools.terrainengine.pysheds_engine import PyShedsEngine
